

import sys
import requests
import pypistats
import json
import yaml
import pyaml
from pprint import pprint
from collections import OrderedDict
from typing import Optional, Tuple, List


def _req(
        path:   str,
        method: str,
        data:   Optional[dict]
) -> Tuple[int, dict]:

    r = {
        'GET': lambda p, *_: requests.get(p),
        'POST': lambda p, d, *_: requests.post(p, data=d),
    }.get(method, lambda *_: None)(path, data)

    return r.status_code, r.json()


# not used anywhere
def post(path: str, data: Optional[dict]):
    return _req(path, 'POST')


def package_download_stats(package: str):

    recent = json.loads(pypistats.recent(package, format='json')).get('data', {})
    overall = json.loads(pypistats.overall(package, mirrors=True, format='json')).get('data', [])[0]

    downloads = {
        'last_day': recent.get('last_day'),
        'last_week': recent.get('last_week'),
        'last_month': recent.get('last_month'),
        'total': overall.get('downloads'),
    }

    major = pypistats.python_major(package, format='json')
    minor = pypistats.python_minor(package, format='json')

    return downloads


def nullish_add_to_dict(d: dict, key, val):
    if not key or not val:
        return
    d[key] = val


def pypi_page_lookup(packages: List[str]):

    d = {}


    for p in packages:
        r = _req(f'https://pypi.org/pypi/{p}/json', 'GET', {})[1]
        # print(r, type(r))
        info = r.get('info', {})

        data = {
            'description': info.get('description')[:200],
            'license': info.get('license'),
            'downloads': package_download_stats(p),
            'package_version': info.get('version'),
            'python_version': '.'.join(map(lambda x: str(x), sys.version_info)),
            'last_releases': '; '.join(list(r.get('releases').keys())[-1:-6:-1])
        }

        nullish_add_to_dict(data, 'name', info.get('name'))
        nullish_add_to_dict(data, 'author', info.get('author'))
        nullish_add_to_dict(data, 'mail', info.get('author_email'))
        nullish_add_to_dict(data, 'summary', info.get('summary'))

        if bugtrack := info.get('bugtrack_url'):
            data['bugs'] = bugtrack

        if page := info.get('home_page'):
            data['page'] = page

        if maintainer := info.get('maintainer'):
            data['maintainer'] = maintainer

        if maintainer_email := info.get('maintainer_email'):
            data['maintainer_email'] = maintainer

        if docs_page := info.get('docs_url'):
            data['docs_page'] = docs_page

        if package_page := info.get('package_url'):
            data['package_page'] = package_page

        d[p] = data

    return d


def show_stats(*packages: str):
    for p in packages:

        p_info = pypi_page_lookup(p)
        p_ordered = OrderedDict(p_info)

        pyaml.dump(p_ordered, sys.stdout,
                         indent=3, width=120, vspacing=[1, 0])
        print()




import re
import argparse


ARGS = {
    'operation': {
        'type': str,
        'metavar': 'OP',
        'nargs': 1,
        'help': 'Operation to commit',
    },
    'packages': {
        'type': str,
        'metavar': 'PACKAGE',
        'nargs': '+',
        'help': 'Which packages to operate on',
    },
}


def parse_args() -> dict:

    parser = argparse.ArgumentParser(description='pips')

    for op, kwargs in ARGS.items():
        parser.add_argument(op, **kwargs)

    args_parsed = vars(parser.parse_args())
    return {
        'op': args_parsed.get('operation', [])[0],
        'packages': args_parsed.get('packages'),
    }


def stdout(*s: str):
    print('I like :beer: and [green underline] this site')


def log(*s: str):
    stdout('[PIPS]', s)


def agree_to_accord():
    res = input('Would you like to continue')


def parse_cmd_result(out, err, status, op, verbose):
    # TODO: Add parser for Uninstall
    if verbose:
        print()
        print(f'{op[0].upper() + op[1:]} Successful.')
        for line in out.split('\n'):
            line_s: [str] = line.split(' ')
            if 'Successfully install' in line:
                print('+', line_s[1][0].upper() + line_s[1][1:], line_s[2].replace('-', ' '))
            elif 'already' in line:

                # informs of dependency
                # if 'from' in line_s[-3]:
                #     print(line_s[-2].partition('->')[2].rstrip(')'), 'dep:')

                print(
                    '+',
                    re.split(r'(>|<|=)=', line_s[3])[0],
                    '==',
                    line_s[-1].lstrip('(').rstrip(')'), '@', line_s[5])



import os
import json
import toml
from superprocessor import cmd


PIPS_PATH = os.path.expanduser(
    '~/.pips'
)

CREDENTIALS_PATH = os.path.join(
    PIPS_PATH, 'credentials'
)


def init_pips():
    if not os.path.exists(PIPS_PATH):
        cmd('mkdir', PIPS_PATH)
    if not os.path.exists(CREDENTIALS_PATH):
        name = input('Org Name: ')
        pips_id = input('PIPS id: ')
        with open(CREDENTIALS_PATH, 'w+') as creds_f:
            toml.dump({'credentials': {'name': name, 'pips_id': pips_id}}, creds_f)
        # cmd('echo', f'"{creds_toml}"', '>', CREDENTIALS_PATH, log=True)


def verify_action():
    res = input('Continue? [y/N]\n -> ')
    if 'y' in res:
        return True
    print('Bye.')
    return False



import sys
import os.path
from typing import List, Union
from superprocessor import cmd

from pips.crawl import show_stats
from pips.cli import parse_cmd_result
from pips.verifier import init_pips, verify_action


class PIP:

    default_path = [
        sys.executable,
        '-m',
        'pip',
    ]

    def __init__(self):
        init_pips()

    def do(self,
           op:              str,
           packages:        List[str],
           executable_path: Union[List[str], None] = None,
           verbose:         bool = True):
        executable_path = (
                executable_path or self.default_path
        )

        show_stats(packages)

        stmt = [
            * self.default_path,
            op,
            * packages,
        ]

        if op in [
            'uninstall'
        ]:
            stmt.append('--yes')

        if verify_action():
            out, err, status = cmd(*stmt, log=False)
            parse_cmd_result(out, err, status, op, verbose=True)




#!/usr/bin/env python
import re
import os
from setuptools import setup, find_packages

PACKAGE_NAME = 'pipsamo'

reqs = ['']

for _ in range(2):
    try:
        reqs = [
            lambda: open('./requirements.txt', 'r').readlines(),
            lambda: open(os.path.join(f'{PACKAGE_NAME}.egg-info', 'requires.txt'), 'r').readlines(),
        ][_]()
        break
    except:
        pass

# get version
with open(os.path.join('pips', 'constants.py'), 'rt') as consts_file:
    version = re.search(r'__VERSION__ = \'(.*?)\'', consts_file.read()).group(1)

setup(
    name=PACKAGE_NAME,
    version='0.3.0',
    description='Pips Installs Packages Securely',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='samo',
    author_email='thesamogroup@gmail.com',
    url='https://github.com/samocorp/pips',
    packages=['pips'],
    package_data={'pips': ['*.pem']},
    install_requires=reqs,
    license='MIT',
    entry_points={
        'console_scripts': ['pips = pips.__main__:run']
    },
    keywords=[
        'python',
        'pip',
        'pips',
        'security',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
